﻿using System.Management;
using System.Net.NetworkInformation;
using System.Net.Sockets;

using anbs_cp.Classes.OsInfos;
using anbs_cp.Enums;
using anbs_cp.Interfaces.OsInfos;

namespace anbs_cp.Classes;

/// <summary>
/// Информация о системе
/// </summary>
public static class OsInfo
{
    /// <summary>
    /// Конструктор
    /// </summary>
    static OsInfo ()
    {
        Windows = GetWindowsInfo();
        Processors = GetProcessors();
        RAM = GetRAMs();
        Videos = GetVideoAdapterInfos();
        Drives = GetDriveInfos();
        Net = GetNet();
    }

    /// <summary>
    /// Информация о Windows
    /// </summary>
    public static IOsWindowsInfo Windows { get; }

    /// <summary>
    /// Список с информацией о процессоре(-ах)
    /// </summary>
    public static List<IOsProcessorInfo> Processors { get; }

    /// <summary>
    /// Количество оперативной памяти в байтах
    /// </summary>
    public static ulong RAM { get; }

    /// <summary>
    /// Видеоадаптеры
    /// </summary>
    public static List<IOsVideoAdapterInfo> Videos { get; }

    /// <summary>
    /// Диски
    /// </summary>
    public static List<IOsDriveInfo> Drives { get; set; }

    public static List<IOsNetInfo> Net { get; set; }

    #region Служебные методы
    /// <summary>
    /// Получает информацию о Windows
    /// </summary>
    /// <returns></returns>
    private static IOsWindowsInfo GetWindowsInfo () =>
        new OsWindowsInfo
        {
            Version = Environment.OSVersion.ToString(),
            Is64 = Environment.Is64BitOperatingSystem,
            PcName = Environment.MachineName,
            WindowsFolder = Environment.SystemDirectory
        };

    /// <summary>
    /// Получение процессоров
    /// </summary>
    /// <returns>Список информации о процессорах</returns>
    private static List<IOsProcessorInfo> GetProcessors ()
    {
        //Создаю результирующий список
        List<IOsProcessorInfo> processorsList = new();

        //Создаю классы менеджмента
        ManagementClass management = new("Win32_Processor");
        ManagementObjectCollection collection = management.GetInstances();

        //Получаю число процессоров
        int processorsCount = collection.Cast<ManagementObject>()
            .Select(static a => a.Properties["ProcessorId"].Value.ToString()).Count();

        //Перебираю массив данных
        for (int ind = 0; ind < processorsCount; ind++)
        {
            //Создаю элемент информации о процессоре
            OsProcessorInfo processor = new()
            {
                Caption = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Caption"].Value.ToString()).ElementAtOrDefault(ind),
                Description = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Description"].Value.ToString()).ElementAtOrDefault(ind),
                DeviceId = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["DeviceID"].Value.ToString()).ElementAtOrDefault(ind),
                Manufacturer = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Manufacturer"].Value.ToString()).ElementAtOrDefault(ind),
                MaxClockSpeed = TypeConverter.StrToInt(collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["MaxClockSpeed"].Value.ToString()).ElementAtOrDefault(ind) ?? "0"),
                Name = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Name"].Value.ToString()).ElementAtOrDefault(ind),
                NumberOfCores = TypeConverter.StrToInt(collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["NumberOfCores"].Value.ToString()).ElementAtOrDefault(ind) ?? "0"),
                NumberOfLogicalProcessors = TypeConverter.StrToInt(collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["NumberOfLogicalProcessors"].Value.ToString()).ElementAtOrDefault(ind) ?? "0")
            };

            //Добавляю в список
            processorsList.Add(processor);
        }

        //Возвращаю список
        return processorsList;
    }

    /// <summary>
    /// Получение общее количество оперативной памяти
    /// </summary>
    /// <returns>Список информации о количестве оперативной памяти</returns>
    private static ulong GetRAMs ()
    {
        //Создаю классы менеджмента
        ManagementClass management = new("Win32_ComputerSystem");
        ManagementObjectCollection collection = management.GetInstances();

        //Получаю результат
        return TypeConverter.StrToUInt64(collection.Cast<ManagementObject>()
            .Select(static a => a.Properties["TotalPhysicalMemory"].Value.ToString()).FirstOrDefault() ?? "0");
    }

    /// <summary>
    /// Получает список видеоадаптеров
    /// </summary>
    /// <returns>Список информации о видеоадаптерах</returns>
    private static List<IOsVideoAdapterInfo> GetVideoAdapterInfos ()
    {
        //Создаю результирующий список
        List<IOsVideoAdapterInfo> videosList = new();

        //Создаю классы менеджмента
        ManagementClass management = new("Win32_VideoController");
        ManagementObjectCollection collection = management.GetInstances();

        //Получаю число адаптеров
        int count = collection.Cast<ManagementObject>()
            .Select(static a => a.Properties["DeviceID"].Value.ToString()).Count();

        //Перебираю массив данных
        for (int ind = 0; ind < count; ind++)
        {
            //Создаю элемент информации об адаптере
            OsVideoAdapterInfo adapter = new()
            {
                Caption = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Caption"].Value.ToString()).ElementAtOrDefault(ind),
                Description = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Description"].Value.ToString()).ElementAtOrDefault(ind),
                DeviceId = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["DeviceID"].Value.ToString()).ElementAtOrDefault(ind),
                AdapterRAM = TypeConverter.StrToInt(collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["AdapterRAM"].Value.ToString()).ElementAtOrDefault(ind) ?? "0"),
                DriverDate = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["DriverDate"].Value.ToString()).ElementAtOrDefault(ind),
                Name = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Name"].Value.ToString()).ElementAtOrDefault(ind),
                CurrentResolution = (TypeConverter.StrToInt(collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["CurrentHorizontalResolution"].Value.ToString()).ElementAtOrDefault(ind) ?? "0"),
                    TypeConverter.StrToInt(collection.Cast<ManagementObject>()
                        .Select(static a => a.Properties["CurrentVerticalResolution"].Value.ToString())
                        .ElementAtOrDefault(ind) ?? "0")),
                SystemName = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["SystemName"].Value.ToString()).ElementAtOrDefault(ind),
                DriverVersion = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["DriverVersion"].Value.ToString()).ElementAtOrDefault(ind),
                InstalledDisplayDrivers = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["InstalledDisplayDrivers"].Value.ToString()).ElementAtOrDefault(ind),
                VideoProcessor = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["VideoProcessor"].Value.ToString()).ElementAtOrDefault(ind)
            };

            //Добавляю в список
            videosList.Add(adapter);
        }

        //Возвращаю список
        return videosList;
    }

    /// <summary>
    /// Получает список дисков
    /// </summary>
    /// <returns>Список информации о дисках</returns>
    private static List<IOsDriveInfo> GetDriveInfos ()
    {
        //Создаю результат
        List<IOsDriveInfo> result = new();

        //Добавление всех HDD/SSD дисков
        result.AddRange(GetHDDs());

        //Добавление всех CD/DVD/BD дисков
        result.AddRange(GetCDRom());

        //Вывожу список
        return result;
    }

    /// <summary>
    /// Получаю список HDD/SSD дисков
    /// </summary>
    /// <returns>Информация обо всех HDD/SSD дисках</returns>
    private static List<IOsDriveInfo> GetHDDs ()
    {
        //Создаю результирующий список
        List<IOsDriveInfo> driveList = new();

        //Создаю классы менеджмента
        ManagementClass management = new("Win32_DiskDrive");
        ManagementObjectCollection collection = management.GetInstances();

        //Получаю число адаптеров
        int count = collection.Cast<ManagementObject>()
            .Select(static a => a.Properties["DeviceID"].Value.ToString()).Count();

        //Перебираю массив данных
        for (int ind = 0; ind < count; ind++)
        {
            //Создаю элемент информации об адаптере
            OsDriveInfo drive = new()
            {
                Type = EDriveType.DtHardDisc,
                Caption = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Caption"].Value.ToString()).ElementAtOrDefault(ind),
                Description = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Description"].Value.ToString()).ElementAtOrDefault(ind),
                DeviceId = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["DeviceID"].Value.ToString()).ElementAtOrDefault(ind),
                Name = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Name"].Value.ToString()).ElementAtOrDefault(ind),
                TotalSize = TypeConverter.StrToUInt64(collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Size"].Value.ToString()).ElementAtOrDefault(ind) ?? "0"),
                Model = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Model"].Value.ToString()).ElementAtOrDefault(ind)
            };

            //Добавляю в список
            driveList.Add(drive);
        }

        //Возвращаю список
        return driveList;
    }

    /// <summary>
    /// Получаю список CD/DVD/BD дисков
    /// </summary>
    /// <returns>Информация обо всех CD/DVD/BD дисках</returns>
    private static List<IOsDriveInfo> GetCDRom ()
    {
        //Создаю результирующий список
        List<IOsDriveInfo> driveList = new();

        //Создаю классы менеджмента
        ManagementClass management = new("Win32_CDROMDrive");
        ManagementObjectCollection collection = management.GetInstances();

        //Получаю число адаптеров
        int count = collection.Cast<ManagementObject>()
            .Select(static a => a.Properties["DeviceID"].Value.ToString()).Count();

        //Перебираю массив данных
        for (int ind = 0; ind < count; ind++)
        {
            //Создаю элемент информации об адаптере
            OsDriveInfo drive = new()
            {
                Type = EDriveType.DtCDRom,
                Caption = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Caption"].Value.ToString()).ElementAtOrDefault(ind),
                Description = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Description"].Value.ToString()).ElementAtOrDefault(ind),
                DeviceId = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["DeviceID"].Value.ToString()).ElementAtOrDefault(ind),
                Name = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Name"].Value.ToString()).ElementAtOrDefault(ind),
                //Ставится 0, чтобы не проверять корректность загрузки и читаемость диска, а также избежать удлинения получения информации
                TotalSize = 0,
                Model = collection.Cast<ManagementObject>()
                    .Select(static a => a.Properties["Manufacturer"].Value.ToString()).ElementAtOrDefault(ind)
            };

            //Добавляю в список
            driveList.Add(drive);
        }

        //Возвращаю список
        return driveList;
    }

    /// <summary>
    /// Получаю информацию о сети интернет
    /// </summary>
    /// <returns>Информация о сети интернет</returns>
    private static List<IOsNetInfo> GetNet ()
    {
        //Создаю результирующий список
        List<IOsNetInfo> netList = new();

        //Получаю информацию о всех сетевых интерфейсах
        foreach (NetworkInterface adapter in NetworkInterface.GetAllNetworkInterfaces())
        {
            //Создаю элемент
            OsNetInfo netInfo = new()
            {
                Name = adapter.Name,
                Description = adapter.Description,
                MacAddress = adapter.OperationalStatus == OperationalStatus.Up ? adapter.GetPhysicalAddress().ToString() : null
            };

            //Получаю IP-адрес
            foreach (UnicastIPAddressInformation info in adapter.GetIPProperties().UnicastAddresses.Where(static info =>
                         info.Address.AddressFamily == AddressFamily.InterNetwork && info.IsDnsEligible))
            {
                if (!string.IsNullOrWhiteSpace(netInfo.IPAddress))
                    netInfo.IPAddress += ";";
                netInfo.IPAddress += info.Address.ToString();
            }

            //Добавляю адаптер
            netList.Add(netInfo);
        }

        //Возвращаю список
        return netList;
    }
    #endregion
}